import os
import sys
import unittest
import logging
from pprint import pformat
from datetime import timedelta, date

from p2.logging import Logger
from p2.scheduler.Shifts import Shifts
from p2.scheduler.Calendar import Calendar
from p2.store.client import Client as StoreClient
from p2.tools.prepare_db import setUpDB, tearDownDB
from postgres.client.sql import SQLit

class TestShifts(unittest.TestCase):

    def setUp(self):
        setUpDB()

        sql = SQLit()
        sql.execute('DELETE FROM shifts')

        log_obj = Logger(name=__name__, level=logging.DEBUG)
        self.logger = log_obj.getLogger()
        self.logger.info("setUp...")

        store_url = os.environ.get('P2_STORE_URL')
        self.client = StoreClient(os.environ.get('P2_STORE_URL'))

        self.shifts = Shifts(tag = 'foo')
        self.logger.info(pformat(self.shifts))
        self.cal = Calendar()

        def tearDown():
            tearDownDB()

    def test_get_session_shift(self):
        start = self.shifts.get_session_shift(duration = 60)
        self.assertIsNotNone(start)
        tomorrow = date.today()
        while ( tomorrow := tomorrow + timedelta(days=1) ):
            if ( self.cal._day_valid(tomorrow) ):
                break
        self.assertEqual(start['day'], tomorrow.day)
        start = self.shifts.get_session_shift(duration = 60)
        self.assertEqual(start['day'], tomorrow.day)
        start = self.shifts.get_session_shift(duration = 60)
        self.assertNotEqual(start['day'], tomorrow.day)

    def test_get_session_shift_in_day(self):
        tomorrow = date.today()
        while ( tomorrow := tomorrow + timedelta(days=1) ):
            if ( self.cal._day_valid(tomorrow) ):
                break
        start = self.shifts.get_session_shift_in_day(tomorrow, 45)
        self.assertIsNotNone(start)
        self.logger.debug(f"Session Start: {start}")

    def test_get_num_sessions_in_day(self):
        tomorrow = date.today()
        while ( tomorrow := tomorrow + timedelta(days=1) ):
            if ( self.cal._day_valid(tomorrow) ):
                break
        start_1 = self.shifts.get_session_shift_in_day(tomorrow, 10)
        start_2 = self.shifts.get_session_shift_in_day(tomorrow, 10)
        self.assertEqual(self.shifts.get_num_sessions_in_day(tomorrow), 2)

    def test_max_sessions_check_in_day(self):
        tomorrow = date.today()
        while ( tomorrow := tomorrow + timedelta(days=1) ):
            if ( self.cal._day_valid(tomorrow) ):
                break
        start = self.shifts.get_session_shift_in_day(tomorrow, 10)
        self.assertIsNotNone(start)
        self.logger.debug(f"Session Start: {start}")
        self.assertEqual(self.shifts.get_num_sessions_in_day(tomorrow), 1)

        start = self.shifts.get_session_shift_in_day(tomorrow, 10)
        self.assertIsNotNone(start)
        self.assertEqual(self.shifts.get_num_sessions_in_day(tomorrow), 2)

        start = self.shifts.get_session_shift_in_day(tomorrow, 10)
        self.assertIsNone(start) # max number of sessions reached...
        self.assertEqual(self.shifts.get_num_sessions_in_day(tomorrow), 2)


if __name__ == '__main__':
    unittest.main()
